/* $Id: BinFraction.java,v 1.1.1.1 2001/05/14 15:55:40 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.base;

import	java.io.Serializable;

/**
 * The abstract superclass for fixed-point rational numbers.
 *
 * Subclasses must provide method |pointShift| which determines the position
 * of fixed-point.
 *
 * The interface of this class is quite limited and serves only for
 * alowing numeric operations between different subtypes of |Number|
 * (rational numbers with different fixed-point position). Most of methods
 * is unfortunatelly not shared in subclasses (although the methods are
 * the same) and the code is simply duplicated. It is so because most of
 * them create a new instance or have more efficient variants for the other
 * operand of the same subtype. It is not clear if sharing is feasible. It
 * would need a virtual constructor (method returning a factory object for
 * creating instances).
 *
 * @author	Karel Skoupy
 * @version	${VERSION}
 * @since	NTS1.0
 *
 * @see		Dimen
 * @see		nts.tfm.FixWord
 */
public abstract class	BinFraction implements Serializable {

    /** Internal 32 bits representation of the rational number */
    protected final int		value;

    /** Constructs a |BinFraction| of certain internal representation value
     * @param	val	the internal representation value
     */
    protected BinFraction(int val) { value = val; }

    /** Position of the fixed-point
     * @return	the position of the fixed-point counted from the right
     */
    protected abstract int	pointShift();

    /** Converts the value of a |BinFraction| to another internal
     * representation with different fixed-point position. It is used for
     * conversion (before operations) between instances of different
     * subclasses.
     * @param	x	|BinFraction| to be converted
     * @param	ps	new fixed-point position
     * @return internal representation for different fixed-point position
     */
    protected static int	makeRepr(BinFraction x, int ps)
	{ return (int) ((long) x.value << ps >> x.pointShift()); }

    /** Makes an internal representation of the product with another
     * |BinFraction|. It is used for multiplication with instance
     * of different subclass.
     * @param	x	|BinFraction| to multiply with
     * @return internal representation of the product with |x|
     */
    protected final int		reprTimes(BinFraction x)
	{ return (int) ((long) value * x.value >> x.pointShift()); }

    /** Makes an internal representation of the quotient by another
     * |BinFraction|. It is used for division by instance
     * of different subclass.
     * @param	x	|BinFraction| to divide by
     * @return internal representation of the quotient by |x|
     */
    protected final int		reprOver(BinFraction x)
	{ return (int) (((long) value << x.pointShift()) / x.value); }

}
